"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Themable = exports.WorkbenchThemeService = exports.COLOR_THEME_SETTING = exports.CUSTOM_EDITOR_COLORS_SETTING = exports.CUSTOM_WORKBENCH_COLORS_SETTING = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const common_1 = require("../common");
const color_1 = require("../common/color");
const color_registry_1 = require("../common/color-registry");
const css_var_1 = require("../common/css-var");
const event_1 = require("../common/event");
const theme_service_1 = require("../common/theme.service");
const theme_store_1 = require("./theme-store");
exports.CUSTOM_WORKBENCH_COLORS_SETTING = 'workbench.colorCustomizations';
exports.CUSTOM_EDITOR_COLORS_SETTING = 'editor.tokenColorCustomizations';
exports.COLOR_THEME_SETTING = 'general.theme';
const tokenGroupToScopesMap = {
    comments: ['comment', 'punctuation.definition.comment'],
    strings: ['string', 'meta.embedded.assembly'],
    keywords: ['keyword - keyword.operator', 'keyword.control', 'storage', 'storage.type'],
    numbers: ['constant.numeric'],
    types: ['entity.name.type', 'entity.name.class', 'support.type', 'support.class'],
    functions: ['entity.name.function', 'support.function'],
    variables: ['variable', 'entity.name.variable'],
};
let WorkbenchThemeService = class WorkbenchThemeService extends ide_core_common_1.WithEventBus {
    constructor() {
        super();
        this.colorRegistry = (0, color_registry_1.getColorRegistry)();
        this.cssVarRegistry = css_var_1.CSSVarRegistry.instance();
        this.colorClassNameMap = new Map();
        this.colorThemeLoaded = new ide_core_common_1.Deferred();
        this.themes = new Map();
        this.themeContributionRegistry = new Map();
        this.themeChangeEmitter = new ide_core_common_1.Emitter();
        this.onThemeChange = this.themeChangeEmitter.event;
        this.parseColorValue = (s, name) => {
            if (s.length > 0) {
                if (s[0] === '#') {
                    return color_1.Color.Format.CSS.parseHex(s);
                }
                else {
                    return s;
                }
            }
            this.logger.error((0, ide_core_common_1.localize)('invalid.default.colorType', '{0} must be either a color value in hex (#RRGGBB[AA] or #RGB[A]) or the identifier of a themable color which provides the default.', name));
            return color_1.Color.red;
        };
        this.listen();
        this.applyPlatformClass();
    }
    async onDidExtensionContributes() {
        const themeMap = this.getAvailableThemeInfos().reduce((pre, cur) => {
            if (!pre.has(cur.themeId)) {
                pre.set(cur.themeId, cur.name);
            }
            return pre;
        }, new Map());
        this.preferenceSettings.setEnumLabels(exports.COLOR_THEME_SETTING, Object.fromEntries(themeMap.entries()));
        if (!this.currentThemeId && !this.currentTheme) {
            this.colorThemeLoaded.resolve();
        }
    }
    get preferenceThemeId() {
        return this.preferenceService.get(exports.COLOR_THEME_SETTING);
    }
    registerThemes(themeContributions, extPath) {
        const disposables = new ide_core_common_1.DisposableCollection();
        disposables.push({
            dispose: () => this.doSetPreferenceSchema(),
        });
        themeContributions.forEach(async (contribution) => {
            const themeExtContribution = { basePath: extPath, contribution };
            const themeId = (0, theme_service_1.getThemeId)(contribution);
            this.themeContributionRegistry.set(themeId, themeExtContribution);
            if (this.preferenceThemeId === themeId) {
                await this.applyTheme(this.preferenceThemeId);
            }
            disposables.push({
                dispose: () => {
                    this.themeContributionRegistry.delete(themeId);
                },
            });
            disposables.push({
                dispose: () => {
                    if (this.currentThemeId === themeId) {
                        this.applyTheme(theme_service_1.DEFAULT_THEME_ID);
                    }
                },
            });
        });
        this.doSetPreferenceSchema();
        return disposables;
    }
    async applyTheme(themeId) {
        if (this.currentThemeId === themeId) {
            return;
        }
        const prevThemeType = this.currentTheme ? this.currentTheme.type : 'dark';
        this.currentThemeId = themeId;
        /**
         * 这里 `applyTheme` 默认应该按照最后一个应用的主题进行加载
         * 但由于 `getTheme` 存在时序问题，例如：
         * 主题 A，E，分别由插件 A，E 贡献
         * 这里先调用 applyTheme(E), 再调用 applyTheme(A)
         * 旧的逻辑由于插件 A ... E 的加载顺序问题，会存在 A 比 E 快加载的情况导致最终应用了错误的主题
         *
         * 故这里增加额外判断，保障最后一个加载的主题应用
         */
        this.latestApplyTheme = themeId;
        const theme = await this.getTheme(themeId);
        if (this.latestApplyTheme !== themeId) {
            return;
        }
        const themeType = (0, theme_service_1.getThemeType)(theme.base);
        this.currentTheme = new Theme(themeType, theme);
        this.currentTheme.setCustomColors(this.colorCustomizations);
        this.currentTheme.setCustomTokenColors(this.tokenColorCustomizations);
        const currentThemeType = this.currentTheme.type;
        this.toggleBaseThemeClass(prevThemeType, currentThemeType);
        this.doApplyTheme(this.currentTheme);
        this.colorThemeLoaded.resolve();
    }
    registerColor(contribution) {
        var _a, _b;
        if (!this.checkColorContribution(contribution)) {
            return;
        }
        const { defaults } = contribution;
        this.colorRegistry.registerColor(contribution.id, {
            light: this.parseColorValue(defaults.light, 'configuration.colors.defaults.light'),
            dark: this.parseColorValue(defaults.dark, 'configuration.colors.defaults.dark'),
            hcDark: this.parseColorValue((_a = defaults.highContrast) !== null && _a !== void 0 ? _a : defaults.dark, 'configuration.colors.defaults.highContrast'),
            hcLight: this.parseColorValue((_b = defaults.highContrastLight) !== null && _b !== void 0 ? _b : defaults.light, 'configuration.colors.defaults.highContrastLight'),
        }, contribution.description);
    }
    // @deprecated 请直接使用sync方法，主题加载由时序保障
    async getCurrentTheme() {
        if (this.currentTheme) {
            return this.currentTheme;
        }
        else {
            const themeData = await this.getTheme(this.currentThemeId);
            return new Theme((0, theme_service_1.getThemeType)(themeData.base), themeData);
        }
    }
    getCurrentThemeSync() {
        if (this.currentTheme) {
            return this.currentTheme;
        }
        else {
            const themeData = this.themeStore.loadDefaultTheme();
            return new Theme((0, theme_service_1.getThemeType)(themeData.base), themeData);
        }
    }
    getColor(colorId) {
        var _a;
        if (!colorId) {
            return undefined;
        }
        if (typeof colorId === 'string') {
            return colorId;
        }
        const color = (_a = this.currentTheme) === null || _a === void 0 ? void 0 : _a.getColor(colorId.id);
        return color ? color_1.Color.Format.CSS.formatHexA(color) : '';
    }
    // 将 colorId 转换成 css 变量
    getColorVar(colorId) {
        if (!colorId) {
            return undefined;
        }
        if (typeof colorId === 'string') {
            return colorId;
        }
        const colorKey = colorId.id;
        return colorKey ? `var(--${colorKey.replace(/\./g, '-')})` : undefined;
    }
    getColorClassNameByColorToken(colorId) {
        if (!colorId) {
            return undefined;
        }
        const id = typeof colorId === 'string' ? colorId : colorId.id;
        if (this.colorClassNameMap.has(id)) {
            return this.colorClassNameMap.get(id);
        }
        const className = `color-token-${(0, ide_core_common_1.uuid)()}`;
        this.styleService.addClass(className, {
            color: this.getColorVar({ id }),
        });
        this.colorClassNameMap.set(id, className);
        return className;
    }
    getAvailableThemeInfos() {
        const themeInfos = [];
        for (const { contribution } of this.themeContributionRegistry.values()) {
            const { label, uiTheme, extensionId } = contribution;
            themeInfos.push({
                themeId: (0, theme_service_1.getThemeId)(contribution),
                name: label,
                base: uiTheme || 'vs',
                extensionId,
            });
        }
        return themeInfos;
    }
    doSetPreferenceSchema() {
        const enums = this.getAvailableThemeInfos().map((info) => info.themeId);
        this.preferenceSchemaProvider.setSchema({
            properties: {
                [exports.COLOR_THEME_SETTING]: {
                    type: 'string',
                    default: enums[0],
                    enum: enums,
                },
            },
        }, true);
    }
    get colorCustomizations() {
        return this.preferenceService.getValid(exports.CUSTOM_WORKBENCH_COLORS_SETTING, {});
    }
    get tokenColorCustomizations() {
        return this.preferenceService.getValid(exports.CUSTOM_EDITOR_COLORS_SETTING, {});
    }
    listen() {
        this.addDispose(this.eventBus.on(event_1.ThemeChangedEvent, (e) => {
            this.themeChangeEmitter.fire(e.payload.theme);
        }));
        this.addDispose(ide_core_common_1.Event.debounce(this.preferenceService.onPreferenceChanged, (_, e) => e, 50)(async (e) => {
            if (e.preferenceName === exports.COLOR_THEME_SETTING) {
                // make sure the theme is registered
                if (this.themeContributionRegistry.has(e.newValue)) {
                    await this.applyTheme(e.newValue);
                }
            }
            if (this.currentTheme) {
                switch (e.preferenceName) {
                    case exports.CUSTOM_WORKBENCH_COLORS_SETTING: {
                        this.currentTheme.setCustomColors(e.newValue);
                        this.doApplyTheme(this.currentTheme);
                        break;
                    }
                    case exports.CUSTOM_EDITOR_COLORS_SETTING: {
                        this.currentTheme.setCustomTokenColors(e.newValue);
                        this.eventBus.fire(new event_1.ThemeChangedEvent({
                            theme: this.currentTheme,
                        }));
                        break;
                    }
                    default:
                        break;
                }
            }
        }));
        this.addDispose(this.colorRegistry.onDidColorChangedEvent((e) => {
            if (this.currentTheme) {
                this.doApplyTheme.apply(this, [this.currentTheme]);
            }
        }));
    }
    checkColorContribution(contribution) {
        if (typeof contribution.id !== 'string' || contribution.id.length === 0) {
            this.logger.error((0, ide_core_common_1.localize)('invalid.id', "'configuration.colors.id' must be defined and can not be empty"));
            return false;
        }
        if (!contribution.id.match(theme_service_1.colorIdPattern)) {
            this.logger.error((0, ide_core_common_1.localize)('invalid.id.format', "'configuration.colors.id' must follow the word[.word]*"));
            return false;
        }
        if (typeof contribution.description !== 'string' || contribution.id.length === 0) {
            this.logger.error((0, ide_core_common_1.localize)('invalid.description', "'configuration.colors.description' must be defined and can not be empty"));
            return false;
        }
        const defaults = contribution.defaults;
        if (!defaults ||
            typeof defaults !== 'object' ||
            typeof defaults.light !== 'string' ||
            typeof defaults.dark !== 'string' ||
            typeof defaults.highContrast !== 'string') {
            this.logger.error((0, ide_core_common_1.localize)('invalid.defaults', "'configuration.colors.defaults' must be defined and must contain 'light', 'dark' and 'highContrast'"));
            return false;
        }
        return true;
    }
    async getTheme(id) {
        const theme = this.themes.get(id);
        if (theme) {
            return theme;
        }
        const themeInfo = this.themeContributionRegistry.get(id);
        if (themeInfo) {
            const { contribution, basePath } = themeInfo;
            return await this.themeStore.getThemeData(contribution, basePath);
        }
        return await this.themeStore.getThemeData();
    }
    doApplyTheme(theme) {
        var _a;
        const registeredColors = this.colorRegistry.getColors();
        const colors = {};
        registeredColors.forEach((c) => {
            const colorId = c.id;
            const color = theme.getColor(colorId);
            colors[colorId] = color ? color.toString() : '';
        });
        const contributions = this.themeContributionProvider.getContributions();
        for (const c of contributions) {
            if ((0, ide_core_common_1.isFunction)(c.onWillApplyTheme)) {
                const contributionColor = c.onWillApplyTheme(theme);
                Object.assign(colors, contributionColor);
            }
        }
        // fallback to editorSelectionBackground when global selectionBackgroun is null.
        if (!theme.getColor(color_registry_1.selectionBackground)) {
            colors[color_registry_1.selectionBackground] = (_a = theme.getColor(color_registry_1.editorSelectionBackground)) === null || _a === void 0 ? void 0 : _a.toString();
            colors[color_registry_1.ktInputSelectionBackground] = colors[color_registry_1.selectionBackground];
        }
        let cssVariables = ':root{';
        let editorCssVariables = '#workbench-editor .monaco-editor {';
        for (const colorKey of Object.keys(colors)) {
            const targetColor = colors[colorKey] || theme.getColor(colorKey);
            if (targetColor) {
                const hexRule = `--${colorKey.replace(/\./g, '-')}: ${targetColor.toString()};\n`;
                if (colorKey.startsWith('vscode')) {
                    editorCssVariables += hexRule;
                }
                else {
                    cssVariables += hexRule;
                }
            }
        }
        this.cssVarRegistry.getVars().forEach((value, key) => {
            cssVariables += `--${key}: ${value};\n`;
        });
        let styleNode = document.getElementById('theme-style');
        if (styleNode) {
            styleNode.innerHTML = editorCssVariables + '}' + cssVariables + '}';
        }
        else {
            styleNode = document.createElement('style');
            styleNode.id = 'theme-style';
            styleNode.innerHTML = editorCssVariables + '}' + cssVariables + '}';
            document.getElementsByTagName('head')[0].appendChild(styleNode);
        }
        if (this.currentTheme) {
            this.eventBus.fire(new event_1.ThemeChangedEvent({
                theme: this.currentTheme,
            }));
        }
    }
    toggleBaseThemeClass(prevThemeType, themeType) {
        const htmlNode = document.getElementsByTagName('html')[0];
        htmlNode.classList.remove((0, theme_service_1.getThemeTypeSelector)(prevThemeType));
        htmlNode.classList.add((0, theme_service_1.getThemeTypeSelector)(themeType));
    }
    applyPlatformClass() {
        const platformClass = ide_core_common_1.isWindows ? 'windows' : ide_core_common_1.isLinux ? 'linux' : 'mac';
        const rootNode = document.getElementsByTagName('body')[0];
        rootNode.classList.add(platformClass);
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(theme_service_1.IThemeStore),
    tslib_1.__metadata("design:type", theme_store_1.ThemeStore)
], WorkbenchThemeService.prototype, "themeStore", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(),
    tslib_1.__metadata("design:type", ide_core_browser_1.Logger)
], WorkbenchThemeService.prototype, "logger", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.PreferenceService),
    tslib_1.__metadata("design:type", Object)
], WorkbenchThemeService.prototype, "preferenceService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.PreferenceSchemaProvider),
    tslib_1.__metadata("design:type", ide_core_browser_1.PreferenceSchemaProvider)
], WorkbenchThemeService.prototype, "preferenceSchemaProvider", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IPreferenceSettingsService),
    tslib_1.__metadata("design:type", Object)
], WorkbenchThemeService.prototype, "preferenceSettings", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(common_1.ICSSStyleService),
    tslib_1.__metadata("design:type", Object)
], WorkbenchThemeService.prototype, "styleService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(common_1.ThemeContributionProvider),
    tslib_1.__metadata("design:type", Object)
], WorkbenchThemeService.prototype, "themeContributionProvider", void 0);
tslib_1.__decorate([
    (0, ide_core_common_1.OnEvent)(ide_core_common_1.ExtensionDidContributes),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", []),
    tslib_1.__metadata("design:returntype", Promise)
], WorkbenchThemeService.prototype, "onDidExtensionContributes", null);
WorkbenchThemeService = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], WorkbenchThemeService);
exports.WorkbenchThemeService = WorkbenchThemeService;
class Themable extends ide_core_common_1.WithEventBus {
    constructor() {
        super();
        this.listenThemeChange();
    }
    get currentTheme() {
        return this.theme;
    }
    // 传入色值ID，主题色未定义时，若useDefault为false则返回undefined
    async getColor(id, useDefault) {
        if (!this.theme) {
            this.theme = await this.themeService.getCurrentTheme();
        }
        return this.theme.getColor(id, useDefault);
    }
    getColorSync(id, useDefault) {
        if (!this.theme) {
            this.theme = this.themeService.getCurrentThemeSync();
        }
        return this.theme.getColor(id, useDefault);
    }
    listenThemeChange() {
        this.themeService.onThemeChange((theme) => {
            this.theme = theme;
            this.style();
        });
    }
    // themeChange时自动调用，首次调用时机需要模块自己判断
    async style() {
        // use theme
    }
}
tslib_1.__decorate([
    (0, di_1.Autowired)(theme_service_1.IThemeService),
    tslib_1.__metadata("design:type", WorkbenchThemeService)
], Themable.prototype, "themeService", void 0);
exports.Themable = Themable;
class Theme {
    constructor(type, themeData) {
        this.colorRegistry = (0, color_registry_1.getColorRegistry)();
        this.defaultColors = Object.create(null);
        this.customColorMap = {};
        this.customTokenColors = [];
        this.type = type;
        this.themeData = themeData;
        this.patchColors();
        this.patchTokenColors();
        this.themeData.loadCustomTokens(this.customTokenColors);
    }
    getColor(colorId, useDefault) {
        const color = this.customColorMap[colorId] || this.getColors()[colorId];
        if (color) {
            return color;
        }
        if (useDefault !== false) {
            return this.getDefault(colorId);
        }
        return undefined;
    }
    defines(color) {
        if (this.customColorMap[color] || this.themeData.colors[color]) {
            return true;
        }
        return false;
    }
    setCustomColors(colors) {
        this.customColorMap = {};
        this.overwriteCustomColors(colors);
        const themeSpecificColors = colors[`[${this.themeData.name}]`];
        if ((0, ide_core_common_1.isObject)(themeSpecificColors)) {
            this.overwriteCustomColors(themeSpecificColors);
        }
    }
    setCustomTokenColors(customTokenColors) {
        this.customTokenColors = [];
        // first add the non-theme specific settings
        this.addCustomTokenColors(customTokenColors);
        // append theme specific settings. Last rules will win.
        const themeSpecificTokenColors = customTokenColors[`[${this.themeData.name}]`];
        if ((0, ide_core_common_1.isObject)(themeSpecificTokenColors)) {
            this.addCustomTokenColors(themeSpecificTokenColors);
        }
        this.themeData.loadCustomTokens(this.customTokenColors);
    }
    addCustomTokenColors(customTokenColors) {
        // Put the general customizations such as comments, strings, etc. first so that
        // they can be overridden by specific customizations like "string.interpolated"
        // eslint-disable-next-line guard-for-in
        for (const tokenGroup in tokenGroupToScopesMap) {
            const group = tokenGroup; // TS doesn't type 'tokenGroup' properly
            const value = customTokenColors[group];
            if (value) {
                const settings = typeof value === 'string' ? { foreground: value } : value;
                const scopes = tokenGroupToScopesMap[group];
                for (const scope of scopes) {
                    this.customTokenColors.push({ scope, settings });
                }
            }
        }
        // specific customizations
        if (Array.isArray(customTokenColors.textMateRules)) {
            for (const rule of customTokenColors.textMateRules) {
                if (rule.scope && rule.settings) {
                    this.customTokenColors.push(rule);
                }
            }
        }
    }
    overwriteCustomColors(colors) {
        // eslint-disable-next-line guard-for-in
        for (const id in colors) {
            const colorVal = colors[id];
            if (typeof colorVal === 'string') {
                this.customColorMap[id] = color_1.Color.fromHex(colorVal);
            }
        }
    }
    patchColors() {
        const colorContributions = this.colorRegistry.getColors();
        for (const colorContribution of colorContributions) {
            const id = colorContribution.id;
            const colorMap = this.themeData.colorMap;
            if (!colorMap[id]) {
                const color = this.colorRegistry.resolveDefaultColor(id, this);
                if (color) {
                    colorMap[id] = color;
                    this.themeData.colors[id] = color_1.Color.Format.CSS.formatHexA(color);
                }
            }
        }
    }
    // 将encodedTokensColors转为monaco可用的形式
    patchTokenColors() {
        // 当默认颜色不在settings当中时，需要补充至颜色值中，默认颜色设置 scope 为 ['']
        // 需要注意的是，后续进行取值时，会依赖数组顺序，初始化后，请不要随意修改
        if (this.themeData.themeSettings.filter((setting) => { var _a; return ((_a = setting.scope) === null || _a === void 0 ? void 0 : _a.length) === 1 && setting.scope[0] === ''; })
            .length === 0) {
            this.themeData.themeSettings.unshift({
                settings: {
                    foreground: this.themeData.colors['editor.foreground']
                        ? this.themeData.colors['editor.foreground'].substring(0, 7)
                        : color_1.Color.Format.CSS.formatHexA(this.colorRegistry.resolveDefaultColor('editor.foreground', this)),
                    background: this.themeData.colors['editor.background']
                        ? this.themeData.colors['editor.background'].substring(0, 7)
                        : color_1.Color.Format.CSS.formatHexA(this.colorRegistry.resolveDefaultColor('editor.background', this)),
                },
                scope: [''],
            });
        }
    }
    // 返回主题内的颜色值
    getColors() {
        if (!this.colorMap) {
            const colorMap = Object.create(null);
            // eslint-disable-next-line guard-for-in
            for (const id in this.themeData.colorMap) {
                colorMap[id] = this.themeData.colorMap[id];
            }
            if (this.themeData.inherit) {
                const baseData = (0, theme_service_1.getBuiltinRules)(this.themeData.base);
                for (const id in baseData.colors) {
                    if (!colorMap[id]) {
                        colorMap[id] = color_1.Color.fromHex(baseData.colors[id]);
                    }
                }
            }
            this.colorMap = colorMap;
        }
        return this.colorMap;
    }
    getDefault(colorId) {
        let color = this.defaultColors[colorId];
        if (color) {
            return color;
        }
        color = this.colorRegistry.resolveDefaultColor(colorId, this);
        this.defaultColors[colorId] = color;
        return color;
    }
}
//# sourceMappingURL=workbench.theme.service.js.map